@@ -1,3 +1,13 @@
+".onUnload" <-
+function (lib, pkg) 
+{
+        .clear.coco.objects(silent = TRUE, pos = .GlobalEnv)
+}
+".Last.lib" <-
+function (lib, pkg) 
+{
+        .clear.coco.objects(silent = TRUE, pos = .GlobalEnv)
+}
 ".after.set.current" <-
 function (old.current = FALSE, result, type = "unconditioned", 
     model = FALSE, push.pop = FALSE, object = .current.coco) 
@@ -276,6 +286,54 @@ function (sh.lib.name = NULL)
 function () 
 sapply(0:127, function(i) parse(text = paste("\"\\", structure(i, 
     class = "octmode"), "\"", sep = ""))[[1]])
+
+
+### ".I"<- function()
+###        c("", "\001", "\002", "\003", "\004", "\005", "\006", "\007",
+###          "\b", "\t", "\n", "\013", "\014", "\r", "\016", "\017", "\020",
+###          "\021", "\022", "\023", "\024", "\025", "\026", "\027", "\030",
+###          "\031", "\000", "\033", "\034", "\035", "\036", "\037", " ",
+###          "!", "\"", "#", "$", "%", "&", "'", "(", ")", "*", "+", ",",
+###          "-", ".", "/", "0", "1", "2", "3", "4", "5", "6", "7", "8", "9",
+###          ":", ";", "<", "=", ">", "?", "@", "A", "B", "C", "D", "E",
+###          "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q",
+###          "R", "S", "T", "U", "V", "W", "X", "Y", "Z", "[", "\\", "]",
+###          "^", "_", "`", "a", "b", "c", "d", "e", "f", "g", "h", "i",
+###          "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u",
+###          "v", "w", "x", "y", "z", "{", "|", "}", "~", "")
+
+### ".J" <-
+### function () 
+### c("", "", "", "", "", "", "", "\a", "\b", "\t", "\n", "\v", 
+###     "\f", "\r", "", "", "", "", "", "", "", "", 
+###     "", "", "", "^ Z", "", "", "", "", "", " ", "!", "\"", 
+###     "#", "$", "%", "&", "'", "(", ")", "*", "+", ",", "-", ".", 
+###     "/", "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", ":", 
+###     ";", "<", "=", ">", "?", "@", "A", "B", "C", "D", "E", "F", 
+###     "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", 
+###     "S", "T", "U", "V", "W", "X", "Y", "Z", "[", "\\", "]", "^", 
+###     "_", "`", "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", 
+###     "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", 
+###     "w", "x", "y", "z", "{", "|", "}", "~", "\177")
+
+".my.ascii" <- function()
+        c("\000","\001","\002","\003","\004","\005","\006","\007",
+          "\010","\011","\012","\013","\014","\015","\016","\017",
+          "\020","\021","\022","\023","\024","\025","\026","\027",
+          "\030","\031","\000","\033","\034","\035","\036","\037",
+          "\040","\041","\042","\043","\044","\045","\046","\047",
+          "\050","\051","\052","\053","\054","\055","\056","\057",
+          "\060","\061","\062","\063","\064","\065","\066","\067",
+          "\070","\071","\072","\073","\074","\075","\076","\077",
+          "\100","\101","\102","\103","\104","\105","\106","\107",
+          "\110","\111","\112","\113","\114","\115","\116","\117",
+          "\120","\121","\122","\123","\124","\125","\126","\127",
+          "\130","\131","\132","\133","\134","\135","\136","\137",
+          "\140","\141","\142","\143","\144","\145","\146","\147",
+          "\150","\151","\152","\153","\154","\155","\156","\157",
+          "\160","\161","\162","\163","\164","\165","\166","\167",
+          "\170","\171","\172","\173","\174","\175","\176","\177")
+
 ".my.dyn.load.open" <-
 function (file) 
 {
